@@ -47,14 +47,36 @@
 (defvar *max-command-alias-depth* 10
   "")
 
+;; XXX: I'd like to just use straight warn, but sbcl drops to the
+;; debugger when compiling so i've made a style warning instead
+;; -sabetts
+(define-condition command-docstring-warning (style-warning)
+  ((command :initarg :command))
+  (:report
+   (lambda (c s)
+     (format s "command ~a doesn't have a docstring" (slot-value c 'command)))))
+
 (defmacro defcommand (name (&rest args) (&rest interactive-args) &body body)
-  "Create a command function and store its interactive hints in *command-hash*."
+  "Create a command function and store its interactive hints in
+*command-hash*. The local variable %interactivep% can be used to check
+if the command was called interactively. If it is non-NIL then it was
+called from a keybinding or from the colon command."
   (check-type name symbol)
+  (let ((docstring (if (stringp (first body))
+                     (first body)
+                     (warn (make-condition 'command-docstring-warning :command name))))
+        (body (if (stringp (first body))
+                  (cdr body) body)))
   `(progn
-     (defun ,name ,args ,@body)
+     (defun ,name ,args
+       ,docstring
+       (let ((%interactivep% *interactivep*)
+	     (*interactivep* nil))
+	 (declare (ignorable %interactivep%))
+	 ,@body))
      (setf (gethash ',name *command-hash*)
            (make-command :name ',name
-                         :args ',interactive-args))))
+                         :args ',interactive-args)))))
 
 (defmacro define-stumpwm-command (name (&rest args) &body body)
   "Deprecated. use `defcommand' instead."
@@ -153,6 +175,39 @@ alias name for the command that is only accessible interactively."
       (throw 'error :abort)))
 
 (defmacro define-stumpwm-type (type (input prompt) &body body)
+  "Create a new type that can be used for command arguments. @var{type} can be any symbol. 
+
+When @var{body} is evaluated @var{input} is bound to the
+argument-line. It is passed to @code{argument-pop},
+@code{argument-pop-rest}, etc. @var{prompt} is the prompt that should
+be used when prompting the user for the argument.
+
+@example
+\(define-stumpwm-type :symbol (input prompt)
+ (or (find-symbol (string-upcase
+		     (or (argument-pop input)
+                         ;; Whitespace messes up find-symbol.
+		         (string-trim \" \"
+		           (completing-read (current-screen)
+					  prompt
+					  ;; find all symbols in the
+					  ;;  stumpwm package.
+					  (let (acc)
+					    (do-symbols (s (find-package \"STUMPWM\"))
+					      (push (string-downcase (symbol-name s)) acc))
+					    acc)))
+                      (throw 'error \"Abort.\")))
+                  \"STUMPWM\")
+     (throw 'error \"Symbol not in STUMPWM package\")))
+
+\(defcommand \"symbol\" (sym) ((:symbol \"Pick a symbol: \"))
+  (message \"~a\" (with-output-to-string (s)
+	          (describe sym s))))
+@end example
+
+This code creates a new type called @code{:symbol} which finds the
+symbol in the stumpwm package. The command @code{symbol} uses it and
+then describes the symbol."
   `(setf (gethash ,type *command-type-hash*)
     (lambda (,input ,prompt)
       ,@body)))
@@ -277,8 +332,8 @@ alias name for the command that is only accessible interactively."
 
 (defun select-group (screen query)
   "Attempt to match string QUERY against group number or partial name."
-  (let (match
-        (num (ignore-errors (parse-integer query))))
+  (let ((num (ignore-errors (parse-integer query)))
+        match)
     (labels ((match (grp)
                (let* ((name (group-name grp))
                       (end (min (length name) (length query))))
@@ -364,25 +419,26 @@ user aborted."
           (apply (command-name cmd-data) args)
         (setf *last-command* command)))))
 
-(defun interactive-command (cmd)
+(defun eval-command (cmd &optional interactivep)
   "exec cmd and echo the result."
   (labels ((parse-and-run-command (input)
              (let* ((arg-line (make-argument-line :string input
                                                   :start 0))
                     (cmd (argument-pop arg-line)))
-               (call-interactively cmd arg-line))))
+               (let ((*interactivep* interactivep))
+		 (call-interactively cmd arg-line)))))
     (multiple-value-bind (result error-p)
         ;; this fancy footwork lets us grab the backtrace from where the
         ;; error actually happened.
         (restart-case
             (handler-bind 
                 ((error (lambda (c)
-                          (invoke-restart 'interactive-command-error
+                          (invoke-restart 'eval-command-error
                                           (format nil "^B^1*Error In Command '^b~a^B': ^n~A~a" 
                                                   cmd c (if *show-command-backtrace* 
                                                             (backtrace-string) ""))))))
               (parse-and-run-command cmd))
-          (interactive-command-error (err-text)
+          (eval-command-error (err-text)
             (values err-text t)))
       ;; interactive commands update the modeline
       (update-all-mode-lines)
@@ -406,7 +462,7 @@ know lisp very well. One might put the following in one's rc file:
   \"split\")
 @end example"
   (loop for i in commands do
-        (interactive-command i)))
+        (eval-command i)))
 
 (defcommand colon (&optional initial-input) (:rest)
   "Read a command from the user. @var{initial-text} is optional. When
@@ -415,4 +471,4 @@ supplied, the text will appear in the prompt."
     (unless cmd
       (throw 'error :abort))
     (when (plusp (length cmd))
-      (interactive-command cmd))))
+      (eval-command cmd t))))
